#include "board.hpp"

using namespace Voltmeter;


void Board::Init()
{
    PreInit();

    // Enable power supply and wait for voltage to stabilize
    Board::PowerEnablePin::SetHigh();

    Utils::DelayMs(100);

    SetRange(Range::_4V);

    AdcTimer::SetMode(AdcTimer::Mode::ClearOnCompareMatch);
    AdcTimer::SetInterrupts(false, false, false);
    AdcTimer::SetClockSource(AdcTimer::ClockSource::ClkDiv8);
    AdcTimer::SetOutputCompareA(Mcu::ClockFrequency / 8 / SamplingFrequency - 1);
    
    Adc::Configure(Adc::Reference::ARef, 6);
    Adc::EnableAutoTrigger(Adc::Trigger::Timer0CompA, true);

    Display::Driver::SetBacklight(false);
    Display::Init();
    Board::Display::Clear();
}


void Board::PreInit()
{
    WatchDog::Init();
    WatchDog::Enable(WatchDog::Timeout::_15ms);

    PowerEnablePin::SetLow();
    PowerEnablePin::ConfigureAsOutput();

    DisplayRegister::Init();
    DisplayRegister::Out(0);

    OnOffButtonPin::ConfigureAsInput(true);
    RangeButtonPin::ConfigureAsInput(true);
    TalkButtonPin::ConfigureAsInput(true);

    SelectAPin::ConfigureAsOutput();
    SelectBPin::ConfigureAsOutput();

    DacMosiPin::ConfigureAsOutput();
    DacCsPin::ConfigureAsOutput();
    DacSckPin::ConfigureAsOutput();

    SdCsPin::ConfigureAsOutput();
    SdMosiPin::ConfigureAsOutput();
    SdMisoPin::ConfigureAsInput(false);
    SdSckPin::ConfigureAsOutput();

    // Unused pins
    Mcu::PortD::Pin0::ConfigureAsInput(true);
    Mcu::PortD::Pin3::ConfigureAsInput(true);
    Mcu::PortD::Pin5::ConfigureAsInput(true);
    Mcu::PortD::Pin6::ConfigureAsInput(true);

    Utils::DelayMs(100);

    Display::Init();
    Board::Display::Clear();
    Board::Display::PutString("-[VOLTMETER400]-");
    Board::Display::Move(0, 1);
    Board::Display::PutString("-[VOLTMETER400]-");

    DacSpi::Init(2);
}


void Board::SetRange(Range const range)
{
    switch (range)
    {
        case Range::_400V:
            SelectAPin::SetHigh();
            SelectBPin::SetLow();
            break;

        case Range::_40V:
            SelectAPin::SetHigh();
            SelectBPin::SetHigh();
            break;

        case Range::_4V:
            SelectAPin::SetLow();
            SelectBPin::SetHigh();
            break;

        case Range::_400mV:
            SelectAPin::SetLow();
            SelectBPin::SetLow();
            break;

    }
}


void Board::SetTimeout(WatchDog::Timeout timeout)
{
    WatchDog::Enable(timeout);
}


void Board::Nop()
{
    WatchDog::Reset();
}


void Board::Loop()
{
    while (true)
    {
        Nop();
    }
}
