#pragma once

#include <stdint.h>
#include <avr/pgmspace.h>
#include "../utils/int_least.hpp"


namespace Devices
{
    template <class DriverT>
    class CharacterDisplay
    {
        public:

            using Driver = DriverT;


            static void Init()
            {
                DriverT::Init();
            }


            static void Move(uint8_t const x, uint8_t const y) 
            {
                DriverT::Move(x, y);
            }


            static void Clear()
            {
                DriverT::Clear();
            }


            static void PutChar(char const ch) 
            {
                DriverT::PutChar(ch);
            }


            static void Pad(char const ch, uint8_t const n)
            {
                for (uint8_t i = 0; i < n; ++i)
                {
                    PutChar(ch);
                }
            }


            static void PutString(char const * const s) 
            {
                PutStringHelper<DriverT>(s, nullptr);
            }


            static void PutStringP(char const * const s) 
            {
                char const * it = s;

                while (true)
                {
                    char ch = pgm_read_byte(it);
                    
                    if (ch == 0)
                    {
                        return;
                    }

                    PutChar(ch);
                    ++it;
                }
            }


            static void PutInt(int x) 
            {
                char buffer[7];

                itoa(x, buffer, 10);
                PutString(buffer);
            }


            static void PutLong(long int x) 
            {
                char buffer[11];

                ltoa(x, buffer, 10);
                PutString(buffer);
            }


            static void PutIntHex(unsigned int x) 
            {
                for (int8_t i = 12; i >= 0; i -= 4) 
                {
                    uint8_t d = (x >> i) & 0x0f;

                    if (d < 10)
                    {
                        PutChar('0' + d);
                    }
                    else
                    {
                        PutChar('a' + d - 10);
                    }
                }
            }


            static void PutLongHex(unsigned long x) 
            {
                for (int8_t i = 28; i >= 0; i -= 4) 
                {
                    uint8_t d = (x >> i) & 0x0f;

                    if (d < 10)
                    {
                        PutChar('0' + d);
                    }
                    else
                    {
                        PutChar('a' + d - 10);
                    }
                }
            }

            template <int bits>
            static void PutIntBin(Utils::UIntLeast<bits> x) 
            {
                for (int8_t i = bits - 1; i >= 0; --i) 
                {
                    if (x & (1 << i))
                    {
                        PutChar('1');
                    }
                    else
                    {
                        PutChar('0');
                    }
                }
            }

        private:

            template <typename DT>
            static void PutStringHelper(char const * const s, decltype(DT::PutString))
            {
                DT::PutString(s);
            }


            template <typename DT>
            static void PutStringHelper(char const * const s, decltype(DT::PutChar))
            {
                char const * it = s;

                while (*it) 
                {
                    DT::PutChar(*it);
                    ++it;
                }
            }
    };
}