#pragma once

#include <stdint.h>
#include "voltage.hpp"
#include "player.hpp"


namespace Voltmeter
{
    class Reader
    {
        public:

            Reader(Player & player);

            void Execute();
            bool IsReading();
            void StopReading(bool finishCurrentFile);

            void ReadMeasurement(Voltage const & acVoltage, Voltage const & dcVoltage);
            void ForceReadMeasurement(Voltage const & acVoltage, Voltage const & dcVoltage);

            void ReadRange(Range const range);
            void ReadAutoRange();

            void ReadCalibration();

        private:

            constexpr static uint8_t const MaxPlaylistLength = 16;

            enum class File
            {
                None = 255,
                NoFile = 254,
                Zero = 0,
                Jeden = 1,
                Dwa = 2,
                Trzy = 3,
                Cztery = 4,
                Piec = 5,
                Szesc = 6,
                Siedem = 7,
                Osiem = 8,
                Dziewiec = 9,
                Dziesiec = 10,
                Jedenascie = 11,
                Dwanascie = 12,
                Trzynascie = 13,
                Czternascie = 14,
                Pietnascie = 15,
                Szesnascie = 16,
                Siedemnascie = 17,
                Osiemnascie = 18,
                Dziewietnascie = 19,
                Dwadziescia = 20,
                Trzydziesci = 30,
                Czterdziesci = 40,
                Piecdziesiat = 50,
                Szescdziesiat = 60,
                Siedemdziesiat = 70,
                Osiemdziesiat = 80,
                Dziewiecdziesiat = 90,
                Sto = 101,
                Dwiescie = 102,
                Trzysta = 103,
                Dziesiata = 110,
                Dziesiate = 111,
                Dziesiatych = 112,
                Setna = 120,
                Setne = 121,
                Setnych = 122,
                Jedna = 131,
                Dwie = 132,
                Wolt = 203,
                Wolty = 204,
                Woltow = 205,
                Wolta = 206,
                Miliwolt = 207,
                Miliwolty = 208,
                Miliwolta = 209,
                Miliwoltow = 210,
                Minus = 220,
                I = 221,
                Przecinek = 222,
                Pol = 223,
                Cisza = 224,
                Poltora = 225,
                DcVoltage = 230,
                AcVoltage = 231,
                EndReadingTOS = 232,
                EndReading = Cisza,
                Working = 233,
                Greetings = 234,
                Zakres400mV = 235,
                Zakres4V = 236,
                Zakres40V = 237,
                Zakres400V = 238,
                ZakresAutomatyczny = 241,
                NapieciePozaZakresem = 239,
                NapiecieUjemnePozaZakresem = 240
            };

            Player * player;
            File playlist[MaxPlaylistLength];
            uint8_t playlistLength;

            Voltage steadyDcVoltage;
            Voltage steadyAcVoltage;
            int steadyReadings;
            bool didReadSteadyVoltage;

            Voltage lastReadAcVoltage;
            Voltage lastReadDcVoltage;

            void AddToPlaylist(File const file);

            void NounAdjectiveForm(int16_t const number, File singular, File pluralNominative, File pluralGenitive);
            void ReadHundreds(int16_t const number);
            void ReadTens(int16_t const number);
            void ReadDigits(int16_t const number, bool isGenitive);
            void ReadNumber(int16_t const number, bool isGenitive);
            void ReadFractionalPart(int16_t number, int16_t denominator);
            void ReadVoltage(bool isAc, Voltage const & voltage);

            void PlayAcVoltageSound();
            void PlayDcVoltageSound();
            void PlayEndReadingSound();
    };
}