#pragma once

#include <stdint.h>
#include "range.hpp"
#include "math_utils.hpp"


namespace Voltmeter
{
    enum class WaveForm
    {
        Ac = 0,
        Dc = 1,
        AcDc = 2
    };
    
    struct Voltage
    {
        enum class Unit
        {
            V,
            mV
        };

        Unit unit;
        int32_t value;
        int16_t denominator;
        uint8_t decimalPoint;

        constexpr Voltage()
            : unit(Unit::V)
            , value(0)
            , denominator(10)
            , decimalPoint(0)
        {}

        constexpr Voltage(Unit const unit, int32_t const value, int16_t const denominator, int8_t const decimalPoint)
            : unit(unit)
            , value(value)
            , denominator(denominator)
            , decimalPoint(decimalPoint)
        {}

        bool IsOverRange() const
        {
            int32_t v = (value > 0) ? value : -value;
            return v >= denominator * 4 / 10;
        }

        bool IsUnderRange() const
        {
            int32_t v = (value > 0) ? value : -value;
            return v < denominator * 4 / 100;
        }

        bool IsEqualTo(Voltage const & other, int32_t const maxDifference) const
        {
            if ((denominator != other.denominator) 
                || (unit != other.unit) 
                || (decimalPoint != other.decimalPoint))
                {
                    return false;
                }

            return (MathUtils::Abs(value - other.value) <= maxDifference);
        }
    };
}